
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.*;
import java.util.ArrayList;

/**
 * The GUI of the client implemented using SWT
 * @see MyClient
 */
public class ClientGUI {
    /**
     * The client to which the GUI will be bound
     */
    static MyClient client;

    /**
     * The container of the news
     */
    static ArrayList news = new ArrayList();

    /**
     * A counter on the list of the news
     */
    static int produced = 0;

    /**
     * The interface
     * @param args nothing
     */
    public static void main(String[] args) {
        final Display display = new Display();
        final Shell mainShell = new Shell(display, SWT.MIN | SWT.CLOSE);
        mainShell.setSize(190, 285);
        mainShell.setText("News Client");
        mainShell.setImage(new Image(display, "Feather.ico"));

        Composite table = new Composite(mainShell, SWT.BORDER);
        table.setBounds(9, 10, 165, 145);

        //Server RMI IP
        Label serverRMIIPLab = new Label(table, SWT.NONE);
        serverRMIIPLab.setBounds(3, 3, 70, 17);
        serverRMIIPLab.setText("Server IP: ");
        final Text serverRMIIPText = new Text(table, SWT.BORDER);
        serverRMIIPText.setBounds(87, 3, 70, 17);
        serverRMIIPText.setTextLimit(15);

        //Server RMI Port
        Label serverRMIPortLab = new Label(table, SWT.NONE);
        serverRMIPortLab.setBounds(3, 26, 78, 17);
        serverRMIPortLab.setText("Server Port: ");
        final Text serverRMIPortText = new Text(table, SWT.BORDER);
        serverRMIPortText.setBounds(87, 26, 70, 17);
        serverRMIPortText.setTextLimit(6);

        //Server RMI Name
        Label serverRMINameLab = new Label(table, SWT.NONE);
        serverRMINameLab.setBounds(3, 49, 82, 17);
        serverRMINameLab.setText("Server Name: ");
        final Text serverRMINameText = new Text(table, SWT.BORDER);
        serverRMINameText.setBounds(87, 49, 70, 17);
        serverRMINameText.setTextLimit(15);

        //RMI IP
        Label RMIAddressIPLab = new Label(table, SWT.NONE);
        RMIAddressIPLab.setBounds(3, 72, 70, 17);
        RMIAddressIPLab.setText("RMI Reg IP: ");
        final Text RMIAddressIPText = new Text(table, SWT.BORDER);
        RMIAddressIPText.setBounds(87, 72, 70, 17);
        RMIAddressIPText.setTextLimit(15);

        //RMI Port
        Label RMIAddressPortLab = new Label(table, SWT.NONE);
        RMIAddressPortLab.setBounds(3, 95, 78, 17);
        RMIAddressPortLab.setText("RMI Reg Port: ");
        final Text RMIAddressPortText = new Text(table, SWT.BORDER);
        RMIAddressPortText.setBounds(87, 95, 70, 17);
        RMIAddressPortText.setTextLimit(6);

        //RMI Name
        Label RMINameLab = new Label(table, SWT.NONE);
        RMINameLab.setBounds(3, 118, 78, 17);
        RMINameLab.setText("RMI Name: ");
        final Text RMINameText = new Text(table, SWT.BORDER);
        RMINameText.setBounds(87, 118, 70, 17);
        RMINameText.setTextLimit(15);

        //News output
        List text = new List(mainShell, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
        text.setBounds(9, 170, 165, 60);

        //Start button
        final Button start = new Button(mainShell, SWT.PUSH);
        start.setText("Start");
        start.setLocation(10, 235);
        start.setSize(50, 20);

        //Stop button
        final Button stop = new Button(mainShell, SWT.PUSH);
        stop.setText("Stop");
        stop.setLocation(66, 235);
        stop.setSize(50, 20);
        stop.setEnabled(false);

        //Quit button
        final Button quit = new Button(mainShell, SWT.PUSH);
        quit.setText("Quit");
        quit.setLocation(122, 235);
        quit.setSize(50, 20);

        //Default values
        serverRMIIPText.setText("localhost");
        serverRMIPortText.setText("3131");
        serverRMINameText.setText("MyServer");
        RMIAddressIPText.setText("localhost");
        RMIAddressPortText.setText("3131");
        RMINameText.setText("MyClient");

        start.addDisposeListener(new DisposeListener(){
            public void widgetDisposed(DisposeEvent event) {
                if(stop.getEnabled() == true && client != null) {
                    client.end();
                    client = null;
                }
            }
        });

        start.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                String message = null;

                if (serverRMIIPText.getText().length() <= 0)
                    message = "RMI Server IP is empty";
                else if (serverRMIPortText.getText().length() <= 0)
                    message = "RMI Server Port is empty";
                else if (serverRMINameText.getText().length() <= 0)
                    message = "RMI Server Name is empty";
                else if (RMIAddressIPText.getText().length() <= 0)
                    message = "RMI IP is empty";
                else if (RMIAddressPortText.getText().length() <= 0)
                    message = "RMI Port is empty";
                else if (RMINameText.getText().length() <= 0)
                    message = "RMI Name is empty";

                if (message != null) {
                    MessageBox mb = new MessageBox(mainShell, SWT.ICON_INFORMATION | SWT.OK);
                    mb.setMessage(message);
                    mb.open();
                    return;
                } else {
                    client = MyClient.getInstance(
                            RMINameText.getText(),
                            serverRMINameText.getText(),
                            "//" + RMIAddressIPText.getText() + ":" + RMIAddressPortText.getText() + "/",
                            "//" + serverRMIIPText.getText() + ":" + serverRMIPortText.getText() + "/",
                            news
                    );

                    if(client == null)
                        return;

                    client.go();

                    start.setEnabled(false);
                    stop.setEnabled(true);
                }
            }
        });

        stop.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                if(client != null) {
                    client.end();
                    client = null;
                }

                stop.setEnabled(false);
            }
        });

        quit.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                if(client != null){
                    client.end();
                    client = null;
                }

                mainShell.close();
            }
        });

        mainShell.open();
        mainShell.setDefaultButton(start);

        while (!mainShell.isDisposed()) {
            if(news.size() > produced) {
                text.add((String)news.get(produced++));
            }

            if (!display.readAndDispatch())
                display.sleep();
        }

        display.close();
    }
}

